#! /usr/bin/perl
# helixcapXval.pl
# For Paper One - Project HELIXCAP
# cross validate the HMM model used
#
# we increase the scope of the HMM source
# with additional sequences from BLAST
# (e-50 cutoff on BLAST, since that appeared
# to be the place where the families stopped)
#
# however, now not all sequences are nonrelated
# we blastclusted to group them
#
# now we must test them cross-validation style
# this means we generate LOTS of HMMs
# one with each cluster removed
# then we test each cluster's seqs using the HMM
# in this way we validate the model
# by getting sensitivity numbers
#
# we get specificity numbers with a PDBminus dataset
#
# algorithm:
# 1) pull in cluster set (switch -c)
# 2) pull in sequences (switch -s)
# 3) generate FULL HMM (and calibrate) (switch -h)
# 4) loop for each cluster:
#    a) generate the HMM minus the cluster (and calibrate)
#    b) test against each sequence in the cluster
#    c) parse results, get statistics
# 5) run the FULL HMM against PDBminus
# 6) calculate statistics and output (switch -o)

use strict;
use Getopt::Long;

#handles to force printing as quickly as feasible
use IO::Handle;
STDIN->autoflush(1);
$| = 1;

my $thisScore;
my $location = "/Applications/HMMer/hmmer-2.3.2.bin.powerpc-apple-osx/binaries";
my $pdbMinus = "/Users/abryan/Desktop/PDB_minus_101207/pdb_minus_101207.fasta";
my $clusterFileName;
my $sequenceFileName;
my $initialFileName;
my $hmmFileName;
my $outputFileName;
my $logFileName;
my $testerFileName;
my $cutoffValue;

Getopt::Long::Configure ('bundling');
GetOptions (	'c=s' => \$clusterFileName,
		's=s' => \$sequenceFileName,
		'i=s' => \$initialFileName,
		'h=s' => \$hmmFileName,
		'o=s' => \$outputFileName,
		'l=s' => \$logFileName,
		't=s' => \$testerFileName,
		'e=i' => \$cutoffValue);

open (CLUSTERFILE, $clusterFileName) or die "program dies: cannot open cluster file \"$clusterFileName\"\n";
open (SEQFILE, $sequenceFileName) or die "program dies: cannot open sequence file \"$sequenceFileName\"\n";
open (OUTFILE, ">$outputFileName") or die "program dies: cannot open output file \"$outputFileName\"\n";
open (LOGFILE, ">$logFileName") or die "program dies: cannot open log file \"$logFileName\"\n";
open (POSFILE, $testerFileName) or die "program dies: cannot open log file \"$testerFileName\"\n";
print "cutoff value $cutoffValue";

# %poshash needs to hold true positives
# best idea: use THE ORIGINAL SET, that is, the sequences used to make $initialFileName
# remove them from the set, compile the HMM, and run it against the original set

my %poshash;
my $posline;
my $currentpos = '';
my $tag = '';
my $ii = 0;
while ($posline = <POSFILE>) {
	# if a header line
	if ($posline =~ /^>/) {
		#done with previous sequence, save it
		if ($tag ne '' && $currentpos ne '') {
			$poshash{$tag} = $currentpos;
			$ii++;
		}
		#reset current seq
		$currentpos = '';
		#get the tag
		$posline =~ /^>gi\|(\d+)\|/;
		$tag = $1;
		print LOGFILE "true positive sequence $tag\n"; 
	}
	else {
		# not a header line
		chomp $posline;
		$currentpos .= $posline;
	}
}
if ($tag ne '') {
	$poshash{$tag} = $currentpos;
}
print "Hi\n";
foreach my $currentKey (keys %poshash) {
	print "key \"$currentKey\" --> sequence $poshash{$currentKey}...\n";
}
print "Bye \n";

print OUTFILE "true positive file parsed. '$ii' entries parsed.\n";
close POSFILE;

# grab the clusters
# make a two-dee array with the sequence IDs

# level 1: @clusterarray, an array of pointers to...
# level 2: @thisCluster, an array of sequence IDs

my @clusterarray;
my $cline;
$ii = 0;
while (my $thisLine = <CLUSTERFILE>) {
	my @thisCluster = split(" ", $thisLine);
#	print "cluster: @thisCluster \n";
	push (@clusterarray, \@thisCluster);
} # END while $cline...

foreach my $currentCluster (@clusterarray) {
	print "cluster: \"@$currentCluster\" \n\n";
}

close CLUSTERFILE;

print "clusterfile parsed \n";

# load sequences
# in the process, associate sequence IDs with sequences in a hash
# %seqhash: (seqID -> sequence)

$tag = '';
my %seqhash = ();
my $currentseq = '';
my $needthis = 0;
$ii = 0;
my $seqline;

while ($seqline = <SEQFILE>) {
	# if a header line
	if ($seqline =~ /^>/) {
		if ($tag ne '') {
			#done with previous sequence, save it
			$seqhash{$tag} = $currentseq;
			$ii++;
		}
		#reset current seq
		$currentseq = '';
		#get the tag
		$seqline =~ /^>gi\|(\d+)\|/;
		$tag = $1;
	}
	else {
		# not a header line
		chomp $seqline;
		$currentseq .= $seqline;
	}
}
if ($tag ne '') {
	$seqhash{$tag} = $currentseq;
}

print OUTFILE "sequence file parsed. '$ii' entries parsed.\n";
close SEQFILE;
system ("sync");
foreach my $currentKey (keys %seqhash) {
 	print "key \"$currentKey\"\n";
 	print "sequence \"$seqhash{$currentKey}\"\n\n";
}

print "Ready to do the full HMM \n";

# generate the full HMM: align it and use the alignment to build the full hmm

## we use hmmalign to align sequences progressively to the already existing hmm
my $alignCommand = "$location/hmmalign -m -o $sequenceFileName.fullalign $initialFileName $sequenceFileName";
print "running $alignCommand:\n";
system($alignCommand);
system ("sync");
## now, rebuild a bigger HMM using that alignment
my $fullHMMCommand = "$location/hmmbuild -s $hmmFileName $sequenceFileName.fullalign";
print "running $fullHMMCommand:\n";
system ($fullHMMCommand);
system ("sync");
my $fullHMMCalibrate = "$location/hmmcalibrate $hmmFileName";
print "running $fullHMMCalibrate:\n";
system ($fullHMMCalibrate);
system ("sync");
print "full hmm calculated, pdb minus run \n";

## run full HMM against PDB minus

my %minusHash;
my $minusOut = "$hmmFileName.minusout";
my $fullHMMRun = "$location/hmmsearch --forward \"$hmmFileName\" \"$pdbMinus\" > \"$minusOut\"";
print "running $fullHMMRun:\n";
system ($fullHMMRun);
system ("sync");
print "pdb minus run calculated, parsing \n";

# and parse it

open (MINUSOUTFILE, $minusOut) or die "Can't open $minusOut\n";
my $linenum = 0;
my $rightSection = 0;
my $outLine;

while ($outLine = <MINUSOUTFILE>) {
	unless ($linenum < 15) {		
		if ($outLine =~ /^\s*Sequence\s*Description\s*Score\s*E-value\s*N\s*$/) {
			$rightSection = 1;
		}
		elsif ($rightSection == 1) {
			if ($outLine =~ /^\s*Parsed\sfor\sdomains/) {
				$rightSection = 2;
			}
			elsif ($outLine =~ /^\s*(\d+)\s+[\-]?\w+/) {
				my $thisSeqName = $1;
				print LOGFILE ".";
				if ($outLine =~ /\s+((-|)\d+.\d+)\s+\d+\w*\d*(e-|.)\d{1,5}\s+\d+\s*$/) {
					$thisScore = $1;
					$minusHash{$thisSeqName} = $thisScore;
					print LOGFILE "MinusHash $thisSeqName $thisScore \n";
				}
				else {
					print LOGFILE "Problem parsing output file $minusOut line $linenum \n";
					print LOGFILE "at $thisSeqName, we have no proper score \n";
				}
			} # END elsif of parsed for domains
		} # END if ($outLine)...
	} # END unless
	$linenum++;
}
close MINUSOUTFILE;
print "pdb minus negative set parsed, doing experiment set \n";

# generate the FASTA file to make the alignment and HMM from
# it contains all the sequences in the HMM feedstock fasta file EXCEPT the ones from POSFILE
my $plusFasta = "$hmmFileName.plus.fasta";
open (PLUSFASTAFILE, ">$plusFasta") or die "Can't open $plusFasta\n";
foreach my $currentKey (sort(keys %seqhash)) {
	my $flag = 0;
	print "loop $currentKey \n";
	foreach my $currentItem (keys %poshash) {
		if ($currentItem eq $currentKey) {
			$flag = 1;
			print "Flag set for $currentItem $currentKey\n";
		}
	}
	print "idiot check \"$currentKey\", $flag\n";
#	if ($currentKey =~ /\d+/) {
		unless ($flag == 1) {
			# print sequence to temp file
			print PLUSFASTAFILE ">$currentKey\n";
#			my @seventies = unpack("A70" x (length($seqhash{$currentKey})/70), $seqhash{$currentKey});
#			foreach my $thisSeventy (@seventies) {
#				print PLUSFASTAFILE "$thisSeventy\n";
#			}
			print "key: $currentKey\n";
			print "seq: $seqhash{$currentKey}\n";
			print PLUSFASTAFILE $seqhash{$currentKey};
			print PLUSFASTAFILE "\n";
#		}
	}
} # END foreach my $currentKey
close PLUSFASTAFILE;
system("sync");
my $plusAlign = "$hmmFileName.plusalign";
## we use hmmalign to align sequences progressively to the already existing hmm
my $plusAlignCommand = "$location/hmmalign -m -o \"$plusAlign\" \"$initialFileName\" \"$plusFasta\"";
print "running $plusAlignCommand:\n";
system($plusAlignCommand);
## now, rebuild a bigger HMM using that alignment

my $plusHMMFilename = "$hmmFileName.plus";
my $plusHMMCommand = "$location/hmmbuild -s $plusHMMFilename $plusAlign";
print "running $plusHMMCommand:\n";
system ($plusHMMCommand);
system ("sync");
my $plusHMMCalibrate = "$location/hmmcalibrate $plusHMMFilename";
print "running $plusHMMCalibrate:\n";
system ($plusHMMCalibrate);
system ("sync");

my $plusIn = "$hmmFileName.plus.in";
open (TEMPPLUSFILE, ">$plusIn") or die "Can't open $plusIn\n";
foreach my $currentPlusSeq (keys %poshash) {
	print TEMPPLUSFILE ">$currentPlusSeq\n";
	print TEMPPLUSFILE "$poshash{$currentPlusSeq}\n";
	print TEMPPLUSFILE "\n";
}
close TEMPPLUSFILE;
system ("sync");
my $plusOut = "$hmmFileName.plus.out";
my $plusCmd = "$location/hmmsearch --forward $plusHMMFilename $plusIn > $plusOut";
print "running $plusCmd:\n";
system ($plusCmd);
system ("sync");
# parse the result

my %plushash;
open (PLUSOUTFILE, $plusOut) or die "Can't open $plusOut\n";
$linenum = 0;
$rightSection = 0;
$outLine;

while ($outLine = <PLUSOUTFILE>) {
	print $outLine;
	unless ($linenum < 15) {		
		if ($outLine =~ /^\s*Sequence\s*Description\s*Score\s*E-value\s*N\s*$/) {
			$rightSection = 1;
			print "RIGHTSECTION = 1";
		}
		elsif ($rightSection == 1) {
			if ($outLine =~ /^\s*Parsed\sfor\sdomains/) {
				$rightSection = 2;
				print "RIGHTSECTION = 2";
			}
			elsif ($outLine =~ /^\s*(\d+)\s+[\-]?\w+/) {
				print "--> ";
				my $thisSeqName = $1;
				my @thisSplitLine = split('\s+', $outLine);
				print "Score @thisSplitLine[1] ";
				if (@thisSplitLine[1] >= $cutoffValue) {
					$thisScore = @thisSplitLine[1];
					$plushash{$thisSeqName} = $thisScore;
					print "PlusHash $thisSeqName $thisScore \n";
				}
				if ($outLine =~ /^s*\d+\s+(-?\d+(\.\d+)?)\s+\d+(\.\d+)?(e-\d+)?\s+\d+\s*$/) {
					print "Regexp worked \n";
				}
			} # END elsif of parsed for domains
		} # END if ($outLine)...
	} # END unless
	$linenum++;
}
close PLUSOUTFILE;

foreach my $plusKey (keys %plushash) {
	print "$plusKey --> $plushash{$plusKey} \n";
}


# for calculations, count the number of pdbMinus seqs

my $numTrueNeg = 0;
open (PDBMINUSFILE, $pdbMinus) or die "Can't open $pdbMinus\n";
foreach my $minusLine (<PDBMINUSFILE>) {
	# if a header line
	if ($minusLine =~ /^>/) {
		$numTrueNeg ++;
	}
}
close PDBMINUSFILE;

print "pdb minus parsed as negative and experiment, now to do cross validation \n";

$ii = 0;
my %scoreHash;

# loop for each cluster
foreach my $currentCluster (@clusterarray) {

### generate HMM minus cluster
### do a foreach on the sequence hash
### check if the key is in the array @$currentCluster
### if it isn't, print that sequence to temp file
### use temp file to make hmm with hmmbuild, hmmcalibrate

	my $tempFile = "cluster-$ii.fasta";
	open (TEMPFILE, ">$tempFile") or die "Can't open $tempFile\n";
	foreach my $currentKey (sort(keys %seqhash)) {
		my $flag = 0;
		foreach my $currentItem (@$currentCluster) {
			if ($currentItem == $currentKey) {
				$flag = 1;
			}
		}
		unless ($flag == 1) {
			# print sequence to temp file
			print TEMPFILE ">$currentKey\n";
			print TEMPFILE "$seqhash{$currentKey}\n";
			print TEMPFILE "\n";
		}
	} # END foreach my $currentKey
	close TEMPFILE;
system("sync");
	my $tempAlign = "cluster-$ii.partalign";
	my $tempAlignCommand = "$location/hmmalign -m -o $tempAlign $initialFileName $tempFile";
	print "running $tempAlignCommand:\n"; 
	system($tempAlignCommand);
system ("sync");
	my $tempHMM = "cluster-$ii.hmm";
	my $buildThis = "$location/hmmbuild -s $tempHMM $tempAlign";
	print "running $buildThis:\n";
	system ($buildThis);
system ("sync");
	my $calibrateThis = "$location/hmmcalibrate $tempHMM";
	print "running $calibrateThis:\n";
	system ($calibrateThis);
system ("sync");
### test against each sequence in the cluster
### run hmmsearch against each sequence (loop using foreach on $@currentCluster)
### parse results, get stats
	my $tempTest = "cluster-test-$ii.fasta";
	my $tempOut = "cluster-test-$ii.out";
	open (TESTFILE, ">$tempTest") or die "Can't open $tempTest\n";
	foreach my $currentSeq (@$currentCluster) {
		print TESTFILE ">$currentSeq\n";
		print TESTFILE "$seqhash{$currentSeq}\n";
		print TESTFILE "\n";
	} # END foreach my $currentSeq
	close TESTFILE;
system("sync");
	my $searchThis = "$location/hmmsearch --forward $tempHMM $tempTest > $tempOut";
	print "running $searchThis:\n";
	system($searchThis);
system("sync");
	# now all that's left is to parse the output
	open (TESTOUTFILE, $tempOut) or die "Can't open $tempOut\n";
	my $linenum = 0;
	my $rightSection = 0;
	while ($outLine = <TESTOUTFILE>) {
		unless ($linenum < 15) {		
			if (($outLine =~ /Sequence/) && ( $outLine =~ /Description/ )) {
				$rightSection = 1;
				print "RIGHTSECTION 1 \n";
			}
			elsif ($rightSection == 1) {
				if ($outLine =~ /^\s*Parsed\sfor\sdomains/) {
					$rightSection = 2;
					print "RIGHTSECTION 2 \n";
				}
				elsif ($outLine =~ /^\s*(\d+)\s+[\-]?\w+/) {
					my $thisSeqName = $1;
					my @thisSplitLine = split ('\s+', $outLine);
					print "splitline: @thisSplitLine\n";
					if (@thisSplitLine[1] >= $cutoffValue) {
						$thisScore = $thisSplitLine[1];
						$scoreHash{$thisSeqName} = $thisScore;
						print "ScoreHash $thisSeqName $thisScore \n";
					}			
					if ($outLine =~ /\s+((-|)\d+.\d+)\s+\d+\w*\d*(e-|.)\d{1,5}\s+\d+\s*$/) {
						print "regexp worked \n";
					}
				} # END elsif of parsed for domains
			} # END if ($outLine)...
		} # END unless
		$linenum++;
	}
	close TESTOUTFILE;
system("sync");
	print "cross validation parse $ii done \n";
	$ii++;
} # END foreach my $currentCluster

print "calculating stats \n";
print OUTFILE "helixcapXval forward output \n";
#timestamp

(my $sec,my $min,my $hour,my $mday,my $mon,my $year,my $wday, my $yday,my $isdst)=localtime(time);
printf OUTFILE "%4d-%02d-%02d %02d:%02d:%02d\n", $year+1900,$mon+1,$mday,$hour,$min,$sec;
print OUTFILE "\n";
print OUTFILE "inputs and outputs: \n";
print OUTFILE "cluster: $clusterFileName \n";
print OUTFILE "sequence: $sequenceFileName \n";
print OUTFILE "initial: $initialFileName \n";
print OUTFILE "full hmm: $hmmFileName \n";
print OUTFILE "output: $outputFileName \n";
print OUTFILE "log: $logFileName \n";
print OUTFILE "test: $testerFileName \n";
print OUTFILE "hmmer binaries at $location \n";
print OUTFILE "pdbminus: $pdbMinus \n";

print OUTFILE "\n--------\n\n";

# calculate stats and output

my @minusArray;
my $minusN = scalar(%minusHash);
if ($minusN) {
	my $minusSum = 0;
	my $minusAverage_deviation = 0;
	my $minusVariance = 0;
	foreach my $minusKey (keys %minusHash) {
		push (@minusArray, $minusHash{$minusKey});
		$minusSum += $minusHash{$minusKey};
		print LOGFILE "minusHash $minusKey $minusHash{$minusKey}";
	}
	my $minusMean = $minusSum / $minusN;
	foreach (@minusArray) {
	    my $deviation = $_ - $minusMean;
	    $minusAverage_deviation += abs($deviation);
	    $minusVariance += $deviation**2;
	}
	$minusAverage_deviation /= $minusN;
	$minusVariance /= ($minusN - 1);
	my $minusStdDev = sqrt($minusVariance);
	
	my @minusArray = sort { $a <=> $b } @minusArray;
	my $minusMid = int($minusN/2);
	my $minusMedian = ($minusN % 2) ? $minusArray[$minusMid] : ($minusArray[$minusMid] + $minusArray[$minusMid-1])/2;

	print OUTFILE "PDB minus stats: \n";
	print OUTFILE "Mean:               $minusMean \n";
	print OUTFILE "Median:             $minusMedian \n";
	print OUTFILE "Average deviation:  $minusAverage_deviation \n";
	print OUTFILE "Standard deviation: $minusStdDev \n";
	print OUTFILE "\n\n";
}
else {
	print OUTFILE "Zero hits on negative control \n";
}

my @plusArray;
my $plusN = scalar(keys %plushash);
print "PlusN = \"$plusN\" \n";
print LOGFILE "PlusN = $plusN";
if ($plusN > 0) {
	my $plusSum = 0;
	my $plusAverage_deviation = 0;
	my $plusVariance = 0;
	foreach my $plusKey (keys %plushash) {
		push (@plusArray, $plushash{$plusKey});
		$plusSum += $plushash{$plusKey};
		print LOGFILE "plusHash $plusKey $plushash{$plusKey}\n";
		print LOGFILE "@plusArray \n";
	}
	my $plusMean = $plusSum / $plusN;
	foreach (@plusArray) {
	    my $deviation = $_ - $plusMean;
	    $plusAverage_deviation += abs($deviation);
	    $plusVariance += $deviation**2;
	}
	print "PlusN = \"$plusN\"\n";
	print LOGFILE"PlusN = \"$plusN\"";
	$plusAverage_deviation /= $plusN;
	$plusVariance /= ($plusN - 1);
	my $plusStdDev = sqrt($plusVariance);
	
	my @plusArray = sort { $a <=> $b } @plusArray;
	my $plusMid = int($plusN/2);
	my $plusMedian = ($plusN % 2) ? $plusArray[$plusMid] : ($plusArray[$plusMid] + $plusArray[$plusMid-1])/2;

	
	print OUTFILE "PDB plus stats: \n";
	print OUTFILE "N:		   $plusN \n";
	print OUTFILE "Mean:               $plusMean \n";
	print OUTFILE "Median:             $plusMedian \n";
	print OUTFILE "Average deviation:  $plusAverage_deviation \n";
	print OUTFILE "Standard deviation: $plusStdDev \n";
	print OUTFILE "\n\n";

	# Now also look at which were which
	#keys %plushash contains those which were detected
	#keys %poshash contains those which were true positives
	# find only elements in @A and not in %B
	my @plusArrayOnly; # DETECTED but not TRUE POSITIVE, i.e., FALSE POSITIVES
	my @posHashOnly; # TRUE POSITIVE but not DETECTED, i.e., FALSE NEGATIVES
	

	# Find False Positives
	foreach my $item (keys %plushash) {
	    print "checking plushash $item\n";
	    unless ($poshash{$item}) {
	        # it's not in %seen, so add to @aonly
	        push(@plusArrayOnly, $item);
		print "$item was in plusArray but not poshash\n";
	    }
	}
	
	# Find False Negatives
	# find only elements in @A and not in @B
	foreach my $item (keys %poshash) {
	    print "checking poshash $item\n";
	    unless ($plushash{$item}) {
	        # it's not in %seen, so add to @aonly
	        push(@posHashOnly, $item);
		print "$item was in poshash but not plusArray\n";
	    }
	}
	
	# output statistics
	
	my $numTruePos = scalar(keys %plushash);
	my $numFalsePos = scalar(@plusArrayOnly);
	my $numFalseNeg = scalar(@posHashOnly);
	my $totalPos = $numTruePos + $numFalsePos;
	my $totalNeg = $numTrueNeg + $numFalseNeg;
	my $totalTrue = $numTruePos + $numFalseNeg;
	my $totalFalse = $numFalsePos + $numTrueNeg;
	my $totalTotal = $totalPos + $totalNeg;
	my $sensitivity = $numTruePos / ($totalTrue);
	my $specificity = $numTrueNeg / ($totalFalse);
	
	print OUTFILE "Experiment statistics: \n\n";
	print OUTFILE "Result   \tPositive\tNegative\tTotal\n";
	print OUTFILE "Structures\t$numTruePos\t$numFalseNeg\t$totalTrue\n";
	print OUTFILE "PDBminus\t$numFalsePos\t$numTrueNeg\t$totalFalse\n";
	print OUTFILE "Total    \t$totalPos\t$totalNeg\t$totalTotal\n";
	print OUTFILE "\n";
	print OUTFILE "Sensitivity: $sensitivity \n";
	print OUTFILE "Specificity: $specificity \n";
	
}
else {
	print OUTFILE "Zero hits on experiment \n";
}

# do all that for the crossval now

my @scoreArray;
my $scoreN = scalar(keys %scoreHash);
if ($scoreN > 0) {
	my $scoreSum = 0;
	my $scoreAverage_deviation = 0;
	my $scoreVariance = 0;
	$ii = 0;
	foreach my $scoreKey (keys %scoreHash) {
		$ii++;
		push (@scoreArray, $scoreHash{$scoreKey});
		print LOGFILE "@scoreArray \n";
		$scoreSum = $scoreSum + $scoreHash{$scoreKey};
	}
	print LOGFILE "ScoreArray Check $ii\n";
	my $scoreMean = $scoreSum / $scoreN;
	foreach (@scoreArray) {
	    my $deviation = $_ - $scoreMean;
	    $scoreAverage_deviation += abs($deviation);
	    $scoreVariance += $deviation**2;
	}
	$scoreAverage_deviation /= $scoreN;
	$scoreVariance /= ($scoreN - 1);
	my $scoreStdDev = sqrt($scoreVariance);
	my @scoreArray = sort { $a <=> $b } @scoreArray;
	my $scoreMid = int($scoreN/2);
	my $scoreMedian = ($scoreN % 2) ? $scoreArray[$scoreMid] : ($scoreArray[$scoreMid] + $scoreArray[$scoreMid-1])/2;


	print OUTFILE "Cross validation stats: \n";
	print OUTFILE "N:                  $scoreN \n";
	print OUTFILE "Mean:               $scoreMean \n";
	print OUTFILE "Median:             $scoreMedian \n";
	print OUTFILE "Average deviation:  $scoreAverage_deviation \n";
	print OUTFILE "Standard deviation: $scoreStdDev \n";
	print OUTFILE "\n\n";
}
else {
	print OUTFILE "Zero cross validation scores \n";
}
	
print OUTFILE "Cross validation scores:\n";
foreach my $scoreKey (keys %scoreHash) {
	print OUTFILE "$scoreKey --> $scoreHash{$scoreKey} \n"; 
}
	
print OUTFILE "\n\n";

print OUTFILE "PDB minus stats: \n";
foreach my $minusKey (keys %minusHash) {
	print OUTFILE "$minusKey --> $minusHash{$minusKey} \n";
}

print OUTFILE "PDB plus stats: \n";
foreach my $plusKey (keys %plushash) {
	print OUTFILE "$plusKey --> $plushash{$plusKey} \n";
}


close OUTFILE;

my $directory = "$hmmFileName-cluster";
system ("mkdir $directory");
system ("mv cluster-*.* $directory");

print "Done!\n";

